import { screen } from '@testing-library/react';
import { Route, Routes } from 'react-router-dom-v5-compat';
import { render } from 'test/test-utils';

import { PanelProps } from '@grafana/data';
import { getPanelPlugin } from '@grafana/data/test/__mocks__/pluginMocks';
import { config, setPluginImportUtils } from '@grafana/runtime';
import { Dashboard } from '@grafana/schema';
import { getRouteComponentProps } from 'app/core/navigation/__mocks__/routeProps';
import { GrafanaRouteComponentProps } from 'app/core/navigation/types';
import { DashboardLoaderSrv, setDashboardLoaderSrv } from 'app/features/dashboard/services/DashboardLoaderSrv';
import { getDashboardScenePageStateManager } from 'app/features/dashboard-scene/pages/DashboardScenePageStateManager';

import { DashboardReportPage, Props } from './DashboardReportPage';

jest.mock('@grafana/runtime', () => ({
  ...jest.requireActual('@grafana/runtime'),
  setPluginExtensionGetter: jest.fn(),
  getPluginLinkExtensions: jest.fn(),
  getBackendSrv: () => {
    return {
      get: jest.fn().mockResolvedValue({ dashboard: dashboard, meta: { url: '' } }),
    };
  },
  getDataSourceSrv: () => {
    return {
      get: jest.fn().mockResolvedValue({}),
      getInstanceSettings: jest.fn().mockResolvedValue({ uid: 'ds1' }),
    };
  },
  getAppEvents: () => ({
    publish: jest.fn(),
  }),
}));

const loadDashboardMock = jest.fn();

function setup(path = '/my-dash-uid', { routeProps }: { routeProps?: Partial<GrafanaRouteComponentProps> } = {}) {
  const defaultRouteProps = getRouteComponentProps();
  const props: Props = {
    ...defaultRouteProps,
    ...routeProps,
  };

  return render(
    <Routes>
      <Route path={'/:uid'} element={<DashboardReportPage {...props} />} />
    </Routes>,
    {
      historyOptions: {
        initialEntries: [path],
      },
    }
  );
}

const panelPlugin = getPanelPlugin(
  {
    skipDataQuery: true,
  },
  CustomVizPanel
);

config.panels['custom-viz-panel'] = panelPlugin.meta;

setPluginImportUtils({
  importPanelPlugin: (id: string) => Promise.resolve(panelPlugin),
  getPanelPluginFromCache: (id: string) => undefined,
});

setDashboardLoaderSrv({
  loadDashboard: loadDashboardMock,
  // disabling type checks since this is a test util
  // eslint-disable-next-line @typescript-eslint/consistent-type-assertions
} as unknown as DashboardLoaderSrv);

describe('DashboardReportPage', () => {
  beforeEach(() => {
    getDashboardScenePageStateManager().clearState();
    getDashboardScenePageStateManager().clearDashboardCache();
    getDashboardScenePageStateManager().clearSceneCache();
    loadDashboardMock.mockClear();
    loadDashboardMock.mockResolvedValue({ dashboard: dashboard, meta: { slug: '123' } });
    // This is a hack way of mocking the resizeObserver used to calculate the header height
    Object.defineProperty(HTMLDivElement.prototype, 'offsetHeight', { configurable: true, value: 90 });
  });

  it('Can render dashboard with repeating panels', async () => {
    setup();

    await waitForDashboardToRender();

    expect(await screen.findByTitle('First panel')).toBeVisible();
    expect(await screen.findByText('First content')).toBeVisible();

    const variables = dashboard.templating!.list![0].options || [];
    for (let i = 0; i < variables.length; i++) {
      expect(await screen.findByTitle(`Panel ${variables[i].value}`)).toBeVisible();
    }
    expect(await screen.findAllByText('Repeating content')).toHaveLength(variables.length);
  });

  it('Should use URL values to render repeating panels', async () => {
    setup('/my-dash-uid?var-myVar=A');

    await waitForDashboardToRender();

    expect(await screen.findByTitle('First panel')).toBeVisible();
    expect(await screen.findByText('First content')).toBeVisible();

    expect(await screen.findByTitle('Panel A')).toBeVisible();
    expect(await screen.findByText('Repeating content')).toBeVisible();

    expect(screen.queryByTitle('Panel B')).not.toBeInTheDocument();
    expect(screen.queryByTitle('Panel C')).not.toBeInTheDocument();
  });

  it('Should use URL values to render time range', async () => {
    setup('/my-dash-uid?from=1727906400000&to=1727992799999&timezone=Europe/Paris');

    await waitForDashboardToRender();

    // We expect 1 pages + the hidden header used to calculate the header height
    expect(await screen.findAllByText('2024-10-03 00:00:00 +02:00', { exact: false })).toHaveLength(2);
    expect(await screen.findAllByText('2024-10-03 23:59:59 +02:00', { exact: false })).toHaveLength(2);
  });
});

const dashboard: Dashboard = {
  title: 'My cool dashboard',
  uid: 'my-dash-uid',
  schemaVersion: 30,
  version: 1,
  panels: [
    {
      id: 1,
      type: 'custom-viz-panel',
      title: 'First panel',
      options: {
        content: 'First content',
      },
      gridPos: {
        x: 0,
        y: 0,
        w: 10,
        h: 10,
      },
      targets: [],
    },
    {
      id: 2,
      type: 'custom-viz-panel',
      title: 'Panel ${myVar}',
      options: {
        content: 'Repeating content',
      },
      repeat: 'myVar',
      repeatDirection: 'h',
      gridPos: {
        x: 0,
        y: 10,
        w: 10,
        h: 10,
      },
      targets: [],
    },
  ],
  templating: {
    list: [
      {
        current: {
          text: ['$__all'],
          value: ['$__all'],
        },
        includeAll: true,
        multi: true,
        name: 'myVar',
        options: [
          { selected: false, text: 'A', value: 'A' },
          { selected: false, text: 'B', value: 'B' },
          { selected: false, text: 'C', value: 'C' },
        ],
        query: 'A,B,C',
        type: 'custom',
      },
    ],
  },
};

interface VizOptions {
  content: string;
}
interface VizProps extends PanelProps<VizOptions> {}

function CustomVizPanel(props: VizProps) {
  return <div>{props.options.content}</div>;
}

async function waitForDashboardToRender() {
  expect(await screen.findByTitle('Panel A')).toBeInTheDocument();
}
